/*

	This program is free software; you can redistribute it and/or modify

	it under the terms of the GNU General Public License version 2 

	as published by the Free Software Foundation.



	This program is distributed in the hope that it will be useful,

	but WITHOUT ANY WARRANTY; without even the implied warranty of

	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.



	You should have received a copy of the GNU General Public License

	along with this program; if not, write to the Free Software

	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA





	Copyright (C) 2006  Thierry Berger-Perrin <tbptbp@gmail.com>

*/

#ifndef MATH_AABB_H

#define MATH_AABB_H



#include "math_vec.h"

#include "math_utility.h"	// minf/maxf



struct aabb_t {

	vec_t min, max;

	aabb_t() {}

	aabb_t(const vec_t &v1, const vec_t &v2): min(v1), max(v2) {}



	//static aabb_t from_triangle(const triangle_t &t) { return t.get_aabb(); }

	// 'this' will encompass 'this' & b.

	FINLINE void compose(const aabb_t &b) {

		min.x = minf(min.x, b.min.x); min.y = minf(min.y, b.min.y); min.z = minf(min.z, b.min.z);

		max.x = maxf(max.x, b.max.x); max.y = maxf(max.y, b.max.y); max.z = maxf(max.z, b.max.z);

	}



	FINLINE double get_area() const { 

		const vec_t ext(max-min);

		return double(ext.x)*double(ext.y) + double(ext.y)*double(ext.z) + double(ext.x)*double(ext.z); 

	}



	vec_t get_extent() const { return max-min; }

};



#endif

